// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef SYNDICATION_VERSION_H
#define SYNDICATION_VERSION_H

#define SYNDICATION_VERSION_STRING "5.105.0"
#define SYNDICATION_VERSION_MAJOR 5
#define SYNDICATION_VERSION_MINOR 105
#define SYNDICATION_VERSION_PATCH 0
#define SYNDICATION_VERSION ((5<<16)|(105<<8)|(0))

#endif
