/*
    This file is part of the syndication library
    SPDX-FileCopyrightText: 2006 Frank Osterfeld <osterfeld@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef SYNDICATION_RDF_RDF_H
#define SYNDICATION_RDF_RDF_H

#include <syndication/rdf/contentvocab.h>
#include <syndication/rdf/document.h>
#include <syndication/rdf/dublincore.h>
#include <syndication/rdf/dublincorevocab.h>
#include <syndication/rdf/image.h>
#include <syndication/rdf/item.h>
#include <syndication/rdf/literal.h>
#include <syndication/rdf/model.h>
#include <syndication/rdf/modelmaker.h>
#include <syndication/rdf/node.h>
#include <syndication/rdf/nodevisitor.h>
#include <syndication/rdf/parser.h>
#include <syndication/rdf/property.h>
#include <syndication/rdf/rdfvocab.h>
#include <syndication/rdf/resource.h>
#include <syndication/rdf/resourcewrapper.h>
#include <syndication/rdf/rssvocab.h>
#include <syndication/rdf/sequence.h>
#include <syndication/rdf/statement.h>
#include <syndication/rdf/syndicationvocab.h>
#include <syndication/rdf/textinput.h>
#include <syndication/syndication.h>

namespace Syndication
{
/**
 * Syndication's parser for the RDF-based
 * RSS 0.9 and RSS 1.0 formats
 */
namespace RDF
{
}
}

#endif // SYNDICATION_RDF_RDF_H
